/*!@file
** @brief   The enumeration of display monitors and top-level windows.
** @remarks It makes a callback to the member function of the derived class.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
**
** @copyright (C) 2014-2019 Ueta System Design Studio\n
** This file is licensed under the terms of the MIT License.\n
** See <https://opensource.org/licenses/MIT>.
*/
#pragma once

#pragma warning(push)
#pragma warning(disable: 4100)

/*!
** @brief   The enumeration of display monitors.
** @remarks -
*/
class ATL_NO_VTABLE CEnumDisplayMonitors
{
public:

	/*!
	** @brief   Enumerates all display monitors on the desktop.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL EnumDisplayMonitors()
	{
		return ::EnumDisplayMonitors(NULL, NULL, &CEnumDisplayMonitors::MonitorEnumProc, LPARAM(this));
	}

	/*!
	** @brief   The callback function that is called by the EnumDisplayMonitors() function.
	** @param   [in] hMonitor a handle to the display monitor.
	** @returns TRUE to continue the enumeration, or FALSE otherwise.
	** @remarks -
	*/
	virtual BOOL EnumDisplayMonitorsProc(HMONITOR hMonitor) = 0;

private:

	static BOOL CALLBACK MonitorEnumProc(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
	{
		CEnumDisplayMonitors* that = reinterpret_cast<CEnumDisplayMonitors*>(dwData);

		return that->EnumDisplayMonitorsProc(hMonitor);
	}
};

/*!
** @brief   The enumeration of top-level windows.
** @remarks -
*/
class ATL_NO_VTABLE CEnumTopLevelWindows
{
public:

	/*!
	** @brief   Enumerates all top-level windows on the screen.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL EnumTopLevelWindows()
	{
		return ::EnumWindows(&CEnumTopLevelWindows::EnumWindowsProc, LPARAM(this));
	}

	/*!
	** @brief   The callback function that is called by the EnumTopLevelWindows() function.
	** @param   [in] hWnd a handle to the top-level window.
	** @returns TRUE to continue the enumeration, or FALSE otherwise.
	** @remarks -
	*/
	virtual BOOL EnumTopLevelWindowsProc(HWND hWnd) = 0;

private:

	static BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lParam)
	{
		CEnumTopLevelWindows* that = reinterpret_cast<CEnumTopLevelWindows*>(lParam);

		return that->EnumTopLevelWindowsProc(hWnd);
	}
};

#pragma warning(pop)
